﻿using UDFS.App.Database;
using UDFS.App.Stores.Connection;
using UDFS.Shared.Database;

namespace UDFS.App.Controls.Connection
{
    /// <summary>
    /// Control for connecting Ms Access Mdb database.
    /// </summary>
    internal partial class MsAccessMdbConnectionControl : UserControl, IConnectionControl
    {
        public MsAccessMdbConnectionControl()
        {
            InitializeComponent();
            databaseInput.Text = Settings.Default.MsAccessMdbLastDatabase;
        }

        public string TabHeader => "Ms Access (*.mdb)";

        public void SetConnection(IConnectionStore connStore)
        {
            string driver = "Microsoft Access Driver (*.mdb)";
            string source = databaseInput.Text;
            string user = usernameInput.Text;
            IDatabase createDatabase()
            {
                return new MsAccessMdbDatabase(@"Driver={" + driver + "};Dbq=" + databaseInput.Text + ";Uid=" + usernameInput.Text + ";Pwd=" + passwordInput.Text + ";");
            }
            connStore.SetConnection(driver, source, user, createDatabase);
            Settings.Default.MsAccessMdbLastDatabase = source;
            Settings.Default.Save();
        }
        public bool ValidateCredentials()
        {
            if (string.IsNullOrWhiteSpace(databaseInput.Text))
            {
                MessageBox.Show("Není vybrána žádná databáze", "Chyba", MessageBoxButtons.OK, MessageBoxIcon.Error);
                return false;
            }
            return true;
        }
        private void SelectDatabaseButton_Click(object sender, EventArgs e)
        {
            using OpenFileDialog dialog = new()
            {
                Filter = "Ms Access (*.mdb)|*.mdb",
                Multiselect = false,
                CheckFileExists = true,
                CheckPathExists = true,
            };
            if (dialog.ShowDialog() == DialogResult.OK)
            {
                databaseInput.Text = dialog.FileName;
            }
        }
    }
}
