﻿namespace UDFS.App.Database
{
    /// <summary>
    /// Interface providing mechanism for using transactions in communication with database.
    /// </summary>
    internal interface ITransactionalDatabase
    {
        /// <summary>
        /// Begins new database transaction.
        /// </summary>
        /// <exception cref="InvalidOperationException">Thrown when transaction already exists.</exception>
        void BeginTransaction();
        /// <summary>
        /// Commits the database transaction.
        /// </summary>
        /// <exception cref="InvalidOperationException">Thrown when no database transaction exists.</exception>
        void CommitTransaction();
        /// <summary>
        /// Rollbacks the database transaction.
        /// </summary>
        /// <exception cref="InvalidOperationException">Thrown when no database transaction exists.</exception>
        void RollbackTransaction();
        /// <summary>
        /// Ends the database transaction.
        /// </summary>
        /// <exception cref="InvalidOperationException">Thrown when no database transaction exists.</exception>
        void EndTransaction();
    }
}
