﻿using System.Data;
using System.Data.Odbc;
using System.IO.Compression;

namespace UDFS.App.Database
{
    /// <summary>
    /// Database communicating with Ms Access Mdb database.
    /// </summary>
    internal class MsAccessMdbDatabase : TransactionalDatabase, IBackupableDatabase
    {
        /// <summary>
        /// Initializes a new instance of <see cref="MsAccessMdbDatabase"/> class.
        /// </summary>
        /// <param name="connectionString">Connection string to database.</param>
        public MsAccessMdbDatabase(string connectionString) : base(connectionString) { }

        public void CreateBackup(DirectoryInfo backupDirectory)
        {
            OdbcConnectionStringBuilder csb = new(ConnectionString);
            string source = csb["Dbq"]?.ToString() ?? "";
            if (source == "") ReportSystem("ZÁLOHA NEBYLA VYTVOŘENA: Není vybrána složka záloh");
            string backupFolderPath = Path.Combine(backupDirectory.FullName, $"BACKUP_{DateTime.Now:dd-MM-yyyy_HH-mm-ss_}" + Path.GetFileNameWithoutExtension(source));
            Directory.CreateDirectory(backupFolderPath);
            string backupFilePath = Path.Combine(backupFolderPath, Path.GetFileName(source));
            File.Copy(source, backupFilePath, true);
            string zipFilePath = backupFolderPath + ".zip";
            ZipFile.CreateFromDirectory(backupFolderPath, zipFilePath);
            Directory.Delete(backupFolderPath, true);
            ReportSystem($"VYTVOŘENA ZÁLOHA DATABÁZE: {zipFilePath}");
        }

        protected override IDbConnection CreateConnection()
        {
            return new OdbcConnection(ConnectionString);
        }
    }
}
