﻿using System.Collections.ObjectModel;
using UDFS.App.Controls.Connection;
using UDFS.App.Stores.Connection;

namespace UDFS.App.Forms
{
    /// <summary>
    /// Form for displaying connection controls to connect to database.
    /// </summary>
    internal partial class ConnectionForm : Form
    {
        private readonly IConnectionStore _connStore;

        /// <summary>
        /// Initializes a new instance of <see cref="ConnectionForm"/> class.
        /// </summary>
        /// <param name="connStore">Connection store to set connection to.</param>
        public ConnectionForm(IConnectionStore connStore)
        {
            InitializeComponent();
            _connStore = connStore;
            ConnectionControls.CollectionChanged += (s, e) => RefreshConnectionControls();
        }

        /// <summary>
        /// Connection controls available in form.
        /// </summary>
        public ObservableCollection<IConnectionControl> ConnectionControls = new();

        /// <summary>
        /// Refreshes available connection controls in form.
        /// </summary>
        private void RefreshConnectionControls()
        {
            connectionControlsTab.TabPages.Clear();
            foreach (IConnectionControl connectionControl in ConnectionControls)
            {
                TabPage tp = new()
                {
                    Text = connectionControl.TabHeader
                };
                UserControl connUserControl = (UserControl)connectionControl;
                connUserControl.Dock = DockStyle.Fill;
                tp.Controls.Add(connUserControl);
                connectionControlsTab.TabPages.Add(tp);
            }
        }
        private void ConnectButton_Click(object sender, EventArgs e)
        {
            if (connectionControlsTab.SelectedIndex >= 0 && connectionControlsTab.SelectedIndex > ConnectionControls.Count)
            {
                MessageBox.Show("Vyskytla se chyba při načítání údajů připojení.");
                DialogResult = DialogResult.None;
            }
            IConnectionControl connControl = ConnectionControls[connectionControlsTab.SelectedIndex];
            if (!connControl.ValidateCredentials())
            {
                DialogResult = DialogResult.None;
                return;
            }
            DialogResult = DialogResult.OK;
            connControl.SetConnection(_connStore);
        }
    }
}
