﻿using UDFS.App.Services.Connection;
using UDFS.App.Services.Functions;
using UDFS.App.Stores.Connection;
using UDFS.App.Stores.Functions;
using UDFS.Shared.Controls;

namespace UDFS.App.Forms
{
    /// <summary>
    /// Form for displaying available user defined functions.
    /// </summary>
    internal partial class MainForm : Form
    {
        private readonly IConnectionStore _connStore;
        private readonly IFunctionsStore _funtionsStore;
        private readonly ISetConnectionService _setConnectionService;
        private readonly ILoadFunctionsService _loadFunctionsService;

        public MainForm(IConnectionStore connStore, IFunctionsStore functionsStore, ISetConnectionService setConnectionService, ILoadFunctionsService loadFunctionsService)
        {
            InitializeComponent();
            _connStore = connStore;
            _connStore.ConnectionChanged += RefreshConnection;
            _funtionsStore = functionsStore;
            _funtionsStore.ControlsChanged += RefreshFunctions;
            _setConnectionService = setConnectionService;
            _loadFunctionsService = loadFunctionsService;
            RefreshConnection();
            RefreshFunctions();
        }

        /// <summary>
        /// Refreshes connection credentials displayed in form.
        /// </summary>
        private void RefreshConnection()
        {
            userLabel.Text = _connStore.User;
            sourceLabel.Text = _connStore.Source;
            driverLabel.Text = _connStore.Driver;
        }
        /// <summary>
        /// Refreshes available functions in menu.
        /// </summary>
        private void RefreshFunctions()
        {
            functionsMenuItem.DropDownItems.Clear();
            foreach (IFunctionControl functionControl in _funtionsStore.Controls)
            {
                ToolStripMenuItem menuItem = new()
                {
                    Text = functionControl.ControlName,
                };
                menuItem.Click += (s, e) => OpenControl(functionControl);
                functionsMenuItem.DropDownItems.Add(menuItem);
            }
            functionsMenuItem.DropDownItems.Add(new ToolStripSeparator());
            ToolStripMenuItem loadFunctionsItem = new()
            {
                Text = "Načíst funkce",
            };
            loadFunctionsItem.Click += (s, e) => _loadFunctionsService.Load();
            functionsMenuItem.DropDownItems.Add(loadFunctionsItem);
        }
        /// <summary>
        /// Opens control of selected user defined function.
        /// </summary>
        /// <param name="functionControl">Function control to display.</param>
        private void OpenControl(IFunctionControl functionControl)
        {
            bodyArea.Controls.Clear();
            UserControl control = (UserControl)functionControl;
            if (control != null)
            {
                control.Dock = DockStyle.Fill;
                bodyArea.Controls.Add(control);
            }
        }
        private void SetConnection_Click(object sender, EventArgs e)
        {
            _setConnectionService.SetConnection();
        }
        private void UnsetConnection_Click(object sender, EventArgs e)
        {
            _setConnectionService.UnsetConnection();
        }
    }
}
