﻿namespace UDFS.App.Forms
{
    /// <summary>
    /// Form for displaying function progress.
    /// </summary>
    internal partial class ProgressForm : Form
    {
        /// <summary>
        /// Indicates, if form is in state of running function.
        /// </summary>
        public bool IsRunning { get; set; }
        /// <summary>
        /// Indicates if form is in state of function has been cancelled.
        /// </summary>
        public bool IsCancelled { get; set; }

        /// <summary>
        /// Occurs when function cancel is requested.
        /// </summary>
        public event Action? CancelRequested;

        /// <summary>
        /// Initializes a new instance of <see cref="ProgressForm"/> class.
        /// </summary>
        /// <param name="functionName">Name of function.</param>
        public ProgressForm(string functionName)
        {
            InitializeComponent();
            functionNameLabel.Text = functionName;
            SetStateRunning();
            IsRunning = true;
        }

        /// <summary>
        /// Refreshes log text in form.
        /// </summary>
        /// <param name="log">Log text to display.</param>
        public void RefreshLog(string log)
        {
            Invoke(new MethodInvoker(delegate
            {
                logInput.Text = log;
            }));
        }
        /// <summary>
        /// Sets form to state of running function.
        /// </summary>
        public void SetStateRunning()
        {
            progressBar.Style = ProgressBarStyle.Marquee;
            progressBar.ForeColor = Color.Green;
            doneButton.Enabled = false;
            cancelButton.Enabled = true;
            IsRunning = true;
        }
        /// <summary>
        /// Sets form to state of cancelling function.
        /// </summary>
        public void SetStateCancelling()
        {
            progressBar.Style = ProgressBarStyle.Marquee;
            progressBar.ForeColor = Color.Yellow;
            doneButton.Enabled = false;
            cancelButton.Enabled = true;
            IsRunning = true;
        }
        /// <summary>
        /// Sets form to state of function has been cancelled.
        /// </summary>
        public void SetStateCancelled()
        {
            progressBar.Style = ProgressBarStyle.Continuous;
            progressBar.Value = 100;
            progressBar.ForeColor = Color.Yellow;
            doneButton.Enabled = true;
            cancelButton.Enabled = false;
            IsRunning = false;
        }
        /// <summary>
        /// Sets form to state of function has been done.
        /// </summary>
        public void SetStateDone()
        {
            progressBar.Style = ProgressBarStyle.Continuous;
            progressBar.Value = 100;
            progressBar.ForeColor = Color.Green;
            doneButton.Enabled = true;
            cancelButton.Enabled = false;
            IsRunning = false;
        }
        private void CancelButton_Click(object sender, EventArgs e)
        {
            SetStateCancelling();
            IsCancelled = true;
            CancelRequested?.Invoke();
        }
        private void DoneButton_Click(object sender, EventArgs e)
        {
            Close();
        }
        private void ProgressForm_Closing(object sender, FormClosingEventArgs e)
        {
            if (IsRunning)
            {
                MessageBox.Show("Funkce stále běží, okno nelze zavřít.", "Chyba", MessageBoxButtons.OK, MessageBoxIcon.Error);
                e.Cancel = true;
            }
        }
    }
}
