﻿using System.Text;

namespace UDFS.App.Services.Files
{
    /// <summary>
    /// Service for clearing old backup files.
    /// </summary>
    internal class ClearBackupFilesService : IClearFilesService
    {
        private readonly DirectoryInfo _backupFilesDir;
        private readonly int _fileLifetimeDays;

        /// <summary>
        /// Intializes a new instance of <see cref="ClearBackupFilesService"/> class.
        /// </summary>
        /// <param name="backupFilesDir">Backup files directory to clear.</param>
        /// <param name="fileLifetimeDays">File lifetime limit in days.</param>
        public ClearBackupFilesService(DirectoryInfo backupFilesDir, int fileLifetimeDays)
        {
            _backupFilesDir = backupFilesDir;
            _fileLifetimeDays = fileLifetimeDays;
        }

        public void Clear()
        {
            List<FileInfo> oldFiles = new();
            StringBuilder filesList = new();
            _backupFilesDir.Create();
            foreach (FileInfo file in _backupFilesDir.GetFiles("BACKUP_*.zip"))
            {
                if (file.CreationTime < DateTime.Now.AddDays(-_fileLifetimeDays))
                {
                    oldFiles.Add(file);
                    filesList.AppendLine($"{file.CreationTime:dd.MM.yyyy hh:mm:ss}: {file.Name}");
                }
            }
            if (oldFiles.Count == 0) return;

            if (MessageBox.Show($"Ve složce {_backupFilesDir} bylo nalezeno {oldFiles.Count} záloh starších než {_fileLifetimeDays} dnů. \n Odstranit tyto zálohy?", "Upozornění", MessageBoxButtons.YesNo, MessageBoxIcon.Information) == DialogResult.Yes)
            {
                foreach (FileInfo oldFile in oldFiles)
                {
                    oldFile.Delete();
                }
            }

        }
    }
}
