﻿using System.Text;

namespace UDFS.App.Services.Logging
{
    /// <summary>
    /// Service for logging progress and storing to file.
    /// </summary>
    internal class FileLogService : ILogService
    {
        private readonly DirectoryInfo _logsDir;
        private readonly StringBuilder _log;

        public FileLogService(DirectoryInfo logsDir)
        {
            _logsDir = logsDir;
            _log = new StringBuilder();
        }

        public event Action? LogChanged;

        public string Log => _log.ToString();

        public void LogSystem(string log)
        {
            _log.AppendLine($"{GetTimeStamp()}  #  {log}");
            LogChanged?.Invoke();
        }
        public void LogUser(string log)
        {
            _log.AppendLine($"{GetTimeStamp()}  >  {log}");
            LogChanged?.Invoke();
        }
        public void Save()
        {
            _logsDir.Create();
            string fileName = Path.Combine(_logsDir.FullName, $"LOG_{DateTime.Now:dd-MM-yyyy_HH-mm-ss}.log");
            File.WriteAllText(fileName, Log);
        }
        /// <summary>
        /// Creates current formatted timestamp.
        /// </summary>
        /// <returns>Returns formatted current timestamp.</returns>
        private static string GetTimeStamp() => DateTime.Now.ToString("dd.MM.yyyy hh:mm:ss");
    }
}
