﻿using UDFS.Shared.Database;

namespace UDFS.App.Stores.Connection
{
    /// <summary>
    /// Interface providing mechanism for storing connection credentials.
    /// </summary>
    internal interface IConnectionStore
    {
        /// <summary>
        /// Occurs when connection changes.
        /// </summary>
        event Action? ConnectionChanged;

        /// <summary>
        /// Connection driver.
        /// </summary>
        string Driver { get; }
        /// <summary>
        /// Connected source.
        /// </summary>
        string Source { get; }
        /// <summary>
        /// Logged user.
        /// </summary>
        string User { get; }
        /// <summary>
        /// Database creator.
        /// </summary>
        Func<IDatabase>? CreateDatabase { get; }

        /// <summary>
        /// Sets connection credentials.
        /// </summary>
        void SetConnection(string driver, string source, string user, Func<IDatabase> createDatabase);
        /// <summary>
        /// Unsets all connection credentials.
        /// </summary>
        void UnsetConnection();
    }
}
