﻿using System.Data;
using UDFS.Shared.Events;

namespace UDFS.Shared.Database
{
    /// <summary>
    /// Interface providing mechanism for communication with database.
    /// </summary>
    public interface IDatabase : IDisposable
    {
        /// <summary>
        /// Occurs when system message reported.
        /// </summary>
        event ReportEventHandler? SysMessageReported;
        /// <summary>
        /// Occurs when user message reported.
        /// </summary>
        event ReportEventHandler? UsrMessageReported;

        /// <summary>
        /// Runs command in database.
        /// </summary>
        /// <param name="commandText">Command text.</param>
        /// <param name="parameters">Command parameters.</param>
        /// <returns>The number of affected rows.</returns>
        int Execute(string commandText, params object[] parameters);
        /// <summary>
        /// Selects data from database.
        /// </summary>
        /// <param name="commandText">Select command text.</param>
        /// <param name="parameters">Select command parameters.</param>
        /// <returns>The data selected from database.</returns>
        DataTable Select(string commandText, params object[] parameters);
        /// <summary>
        /// Selects one item from database.
        /// </summary>
        /// <typeparam name="T">Result data type.</typeparam>
        /// <param name="commandText">Select command text.</param>
        /// <param name="defaultValue">Default value.</param>
        /// <param name="parameters">Select command parameters.</param>
        /// <returns>Selected item. The default value is returned when DBNull or cast error occurs.</returns>
        T SelectItem<T>(string commandText, T defaultValue, params object[] parameters);
        /// <summary>
        /// Selects list from database.
        /// </summary>
        /// <typeparam name="T">Result data type.</typeparam>
        /// <param name="commandText">Select command.</param>
        /// <param name="defaultValue">Default value.</param>
        /// <param name="parameters">Select command parameters.</param>
        /// <returns>Selected list. The default value is used for item when DBNull or cast error occurs.</returns>
        List<T> SelectList<T>(string commandText, T defaultValue, params object[] parameters);
    }
}
