﻿using UDFS.Shared.Database;
using UDFS.Shared.Events;

namespace UDFS.Shared.Functions
{
    /// <summary>
    /// Interface of user defined function.
    /// </summary>
    public interface IUDFunction
    {
        /// <summary>
        /// Occurs when system message is reported.
        /// </summary>
        event ReportEventHandler? SysMessageReported;
        /// <summary>
        /// Occurs when user message is reported.
        /// </summary>
        event ReportEventHandler? UsrMessageReported;

        /// <summary>
        /// Name of user defined function.
        /// </summary>
        string FunctionName { get; }
        /// <summary>
        /// Indicates if transaction should be used.
        /// </summary>
        bool UseTransaction { get; }
        /// <summary>
        /// Indicates if backup should be created.
        /// </summary>
        bool CreateBackup { get; }
        /// <summary>
        /// Indicates if cancel is pending.
        /// </summary>
        bool IsCancelPending { get; set; }
        /// <summary>
        /// Connected database.
        /// </summary>
        IDatabase Database { get; set; }

        /// <summary>
        /// Runs function.
        /// </summary>
        void Run();
    }
}
