﻿using UDFS.Shared.Database;
using UDFS.Shared.Events;

namespace UDFS.Shared.Functions
{
    /// <summary>
    /// Base abstract class of user defined function.
    /// </summary>
    public abstract class UDFunction : IUDFunction
    {
        private IDatabase? _database;

        public event ReportEventHandler? SysMessageReported;
        public event ReportEventHandler? UsrMessageReported;

        public string FunctionName { get; set; } = "";
        public bool UseTransaction { get; set; } = true;
        public bool CreateBackup { get; set; } = true;
        public bool IsCancelPending { get; set; } = false;
        public IDatabase Database
        {
            get => _database ?? throw new InvalidOperationException("Není připojena žádná databáze");
            set => _database = value;
        }

        public void Run()
        {
            SysMessageReported?.Invoke(this, new ReportEventArgs($"SPUŠTĚNÍ FUNKCE '{FunctionName}'"));
            Start();
            SysMessageReported?.Invoke(this, new ReportEventArgs($"UKONČENÍ FUNKCE '{FunctionName}'"));
        }
        /// <summary>
        /// Reports message.
        /// </summary>
        /// <param name="message">Message to report.</param>
        public void Report(string message)
        {
            UsrMessageReported?.Invoke(this, new ReportEventArgs(message));
        }
        /// <summary>
        /// Checks if cancel is pending. If cancel is pending an exception is thrown.
        /// </summary>
        /// <exception cref="FunctionCancelledException">Exception when cancel is pending.</exception>
        public void CanEnd()
        {
            if (IsCancelPending) throw new FunctionCancelledException();
        }
        /// <summary>
        /// User defined function code.
        /// </summary>
        public abstract void Start();
    }

    /// <summary>
    /// Occurs when function is cancelled.
    /// </summary>
    public class FunctionCancelledException : Exception { }
}
